unit uMain;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  System.Math.Vectors, FMX.StdCtrls, FireDAC.Stan.Option, FMX.Controls3D, FMX.Layers3D,
  FMX.Controls.Presentation, FMX.ScrollBox, FMX.Memo, FireDAC.Stan.Intf,
  FireDAC.Stan.Param, FireDAC.Stan.Error, FireDAC.DatS, FireDAC.Phys.Intf,
  FireDAC.DApt.Intf, Data.DB, FireDAC.Comp.DataSet, FireDAC.Comp.Client,
  FMX.Layouts, FireDAC.Stan.StorageBin, Datasnap.DBClient,
  FireDAC.Stan.StorageJSON;

type
  TfrmMain = class(TForm)
    Memo1: TMemo;
    BufferLayer3D1: TBufferLayer3D;
    FDMemTable1: TFDMemTable;
    FDStanStorageBinLink1: TFDStanStorageBinLink;
    ClientDataSet1: TClientDataSet;
    btnTestFDS: TButton;
    btnTestCDS: TButton;
    btnClear: TButton;
    FDStanStorageJSONLink1: TFDStanStorageJSONLink;
    procedure FormCreate(Sender: TObject);
    procedure btnTestFDSClick(Sender: TObject);
    procedure btnTestCDSClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
  private
    sDataPath: string;
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses
  System.IOUtils, System.Diagnostics;

{$R *.fmx}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
{$IFDEF WIN32}
  sDataPath := IncludeTrailingPathDelimiter(ExtractFilePath(ParamStr(0)));
{$ELSE}
  sDataPath := IncludeTrailingPathDelimiter(TPath.GetDocumentsPath);
{$ENDIF}
end;

procedure TfrmMain.btnTestFDSClick(Sender: TObject);
var
  DataStream: TMemoryStream;
  Stopwatch: TStopwatch;
  PrevTime: integer;
begin
  Memo1.Lines.Add('[TFDMemTable Test] --------------------------------');

  while FDMemTable1.Indexes.Count > 0 do
    FDMemTable1.Indexes.Delete(0);
  FDMemTable1.Close;

  Stopwatch := TStopwatch.StartNew;

  DataStream := TMemoryStream.Create;
  try
    DataStream.LoadFromFile(sDataPath + 'customer.fds');

    Memo1.Lines.Add(Format('File load: %dms', [Stopwatch.ElapsedMilliseconds]));
    PrevTime := Stopwatch.ElapsedMilliseconds;

    DataStream.Position := 0;
    FDMemTable1.LoadFromStream(DataStream, TFDStorageFormat.sfBinary);

    Memo1.Lines.Add(Format('Dataset load (%d record): %dms', [FDMemTable1.RecordCount, Stopwatch.ElapsedMilliseconds - PrevTime]));
    PrevTime := Stopwatch.ElapsedMilliseconds;
  finally
    DataStream.Free;
  end;

  if FDMemTable1.Locate('Company', VarArrayOf(['DevQuest']), []) then
    Memo1.Lines.Add(Format('Search & found data ("%s")', [FDMemTable1.FieldByName('Contact').AsString]));

  Memo1.Lines.Add(Format('Locate WITHOUT index: %dms', [Stopwatch.ElapsedMilliseconds - PrevTime]));
  PrevTime := Stopwatch.ElapsedMilliseconds;

  FDMemTable1.First;

  with FDMemTable1.Indexes.Add do
  begin
    Name     := 'Idx_Company';
    Fields   := 'Company';
    Selected := true;
  end;
  Memo1.Lines.Add(Format('Setting index: %dms', [Stopwatch.ElapsedMilliseconds - PrevTime]));
  PrevTime := Stopwatch.ElapsedMilliseconds;

  if FDMemTable1.Locate('Company', VarArrayOf(['DevQuest']), []) then
    Memo1.Lines.Add(Format('Search & found data ("%s")', [FDMemTable1.FieldByName('Contact').AsString]));

  Memo1.Lines.Add(Format('Locate WITH index: %dms', [Stopwatch.ElapsedMilliseconds - PrevTime]));
  PrevTime := Stopwatch.ElapsedMilliseconds;

  Memo1.Lines.Add('');
end;



procedure TfrmMain.btnTestCDSClick(Sender: TObject);
var
  DataStream: TMemoryStream;
  Stopwatch: TStopwatch;
  PrevTime: integer;
begin
  Memo1.Lines.Add('[TClientDataSet Test] --------------------------------');

  ClientDataSet1.IndexName := '';
  ClientDataSet1.IndexDefs.Clear;
  ClientDataSet1.Close;

  Stopwatch := TStopwatch.StartNew;

  DataStream := TMemoryStream.Create;
  try
    DataStream.LoadFromFile(sDataPath + 'customer.cds');

    Memo1.Lines.Add(Format('File load: %dms', [Stopwatch.ElapsedMilliseconds]));
    PrevTime := Stopwatch.ElapsedMilliseconds;

    DataStream.Position := 0;
    ClientDataSet1.LoadFromStream(DataStream);

    Memo1.Lines.Add(Format('Dataset load (%d record): %dms', [ClientDataSet1.RecordCount, Stopwatch.ElapsedMilliseconds - PrevTime]));
    PrevTime := Stopwatch.ElapsedMilliseconds;
  finally
    DataStream.Free;
  end;

  if ClientDataSet1.Locate('Company', VarArrayOf(['DevQuest']), []) then
    Memo1.Lines.Add(Format('Search & found data ("%s")', [ClientDataSet1.FieldByName('Contact').AsString]));

  Memo1.Lines.Add(Format('Locate WITHOUT index: %dms', [Stopwatch.ElapsedMilliseconds - PrevTime]));
  PrevTime := Stopwatch.ElapsedMilliseconds;

  ClientDataSet1.First;

  ClientDataSet1.IndexDefs.Add('Idx_Company', 'Company', []);
  ClientDataSet1.IndexName := 'Idx_Company';

  Memo1.Lines.Add(Format('Setting index: %dms', [Stopwatch.ElapsedMilliseconds - PrevTime]));
  PrevTime := Stopwatch.ElapsedMilliseconds;

  if ClientDataSet1.FindKey(['DevQuest']) then
    Memo1.Lines.Add(Format('Search & found data ("%s")', [ClientDataSet1.FieldByName('Contact').AsString]));

  Memo1.Lines.Add(Format('FindKey WITH index: %dms', [Stopwatch.ElapsedMilliseconds - PrevTime]));
  PrevTime := Stopwatch.ElapsedMilliseconds;

  Memo1.Lines.Add('');
end;

procedure TfrmMain.btnClearClick(Sender: TObject);
begin
  Memo1.Lines.Clear;
end;

end.
